@@ -44,6 +44,10 @@
 #include <ucommon/string.h>
 #endif
 
+#ifndef _UCOMMON_TYPEREF_H_
+#include <ucommon/typeref.h>
+#endif
+
 #ifndef _UCOMMON_MEMORY_H_
 #include <ucommon/memory.h>
 #endif
@@ -372,6 +376,7 @@ public:
      */
     static int prefix(char *path, size_t size);
 
+    // to be depricated...
     static string_t prefix(void);
 
     /**
